/*

Skript pro zálohu nastavení uživatelského rozhraní pro iNuvio

HARPAGON Software s.r.o.  
Ing.Petr Pokorný

Upozornění
- Proceduru vždy spouštějte jako sa.
- Použití je na vlastní nebezpečí

Použití
-------
Nejprve standardně proveďte upgrade databáze na iNuvio a pohodlně nastavte práva a upravte  prostředí pro každého uživatele v iNuvio. Nechte nějaký čas na otestování nastavení uživatelům.
Uživatelé rutinně stále pracují v HeO V2! V iNuviu si testují a nastavují si svoje pracovní prostředí, popř. se řeší úpravy a nastavení pomocí konzultantů.
Pokud je vše nastavné v iNuviu tak, jak má a uživatelé jsou spokojeni -> příkročte k ostrému přechodu.

Legenda
-------
 - Helios001        - Databáze HeO V2 - ostrá aktuální data
 - Helios001INV     - Databáze HeOiNuvio - testovací data s "naklikaným" nastavení uživatelského prostředí, sestavy, právy atd.

Postup
------
1. Krok - nahrání zálohovací procedury
Proceduru hpx_HPG_Zaloha_Nastaveni_iNuvio nahrajeme do Helios001
 
2. Krok
V Helios001 sputíme proceduru a ozálohujeme nastavení:

EXEC Helios001.dbo.hpx_HPG_Zaloha_Nastaveni_iNuvio
 @IsExport       = 1               -- EXPORT
,@SourceDB       = 'HeliosIN001'   -- z iNuvia bereme tabulky,které chceme ozálohovat
,@DestinationDB  = 'Helios001'     -- Kopírujeme je do staré DB (po překlopení DB se tyto tabulky budou nacházet v "nové databázi" v Helios001INV)
,@Suffix         = '_INVZAL'       -- Tabulky musí mít jiné jméno! (nesmí se jemnovat jako originál) Proto se jim přidá suffix.

3. Krok
Naimportujeme Helios001 do Helios001INV a sputíme iNuvio - provede se standardní přechod na iNuvio.

4. Krok
Obnovení nastavení iNuvia, tak jak bylo připravené v rámci testovacího provozu.

EXEC Helios001.dbo.hpx_HPG_Zaloha_Nastaveni_iNuvio
 @IsExport       = 0               -- IMPORT 
,@SourceDB       = 'HeliosIN001'  
,@DestinationDB  = 'HeliosIN001'     
,@Suffix         = '_INVZAL'       -- Suffix tabulek, ve kterých jsou zálohy.

*/

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[hpx_HPG_Zaloha_Nastaveni_iNuvio]
 @IsExport      BIT = 0 
,@SourceDB      NVARCHAR(50) =  NULL -- Defaultně se bere aktuální databáze 
,@DestinationDB NVARCHAR(50) = 'Helios001' 
,@Suffix        NVARCHAR(10) = '_INVZAL' 
,@TableString   NVARCHAR(max) = NULL  -- Pokud je prázdné naataví se default viz dále. Seznam tabulek, které se zálohují/obnovují ZÁLEŽÍ na pořadí!!! Jsou tam zohledněné FK a PK!!
AS
BEGIN

IF @TableString IS NULL -- Defaultní nastavení tabulek pro archivaci
BEGIN
 SET @TableString	= ''
 SET @TableString  +=	N'TabIkony,TabIkonyProAkce,TabIkonyProPrehledy,TabIkonyProSoudky'
 SET @TableString  +=	N',TabUziv,TabzivAtr,TabUzivAtrPrac,TabUzivStrom,TabUzivAtrPracHod,TabUserCfg'
 SET @TableString  +=	N',TabDefTabUzivTabulka,TabDefTabUzivConstraint,TabDefTabUzivAtributy,TabDefTabUzivPrava,TabDefTabUzivZalozky'
 SET @TableString  +=   N',TabInfoPanel,TabInfoPanelDetail,TabInfoPanelDetailUsers,TabOblibeneZkratky,TabRibbonDefinition'
END

    DECLARE 
     @CMDSQL NVARCHAR(max)
	,@CMDSQL2 NVARCHAR(max)
    ,@CMDSet NVARCHAR(max)
	,@CMDOn NVARCHAR(MAX)
    ,@Identity INT = 0
    ,@TableName NVARCHAR(100)
    ,@TableNameSfx NVARCHAR(100)
    ,@ExistujeTabulka BIT
   
-- Šablony
    ,@Sablona_DB NVARCHAR(30) = N'#DBS#'
    ,@Sablona_POLE NVARCHAR(max) = N'SELECT @CMDX ='''',@IdentityX = 0,@CMDXSet ='''',@CMDXOn ='''';
		SELECT  @CMDX +=  IIF((A.is_identity = 1 ) AND @IDDisableX = 1,'''',IIF(LEN(@CMDX) =0,'''','','') + A.Name) , 
		@IdentityX += A.is_identity,@CMDXSet += IIF(ISNULL(A.is_primary_key,0) = 1 OR A.is_identity = 1 ,'''',IIF(LEN(@CMDXSet) =0,'''','','') +''DS.''+ A.Name + ''= SR.''+ A.Name),
		@CMDXOn += IIF(ISNULL(A.is_primary_key,0) = 1,IIF(LEN(@CMDXOn) =0,'''','' AND '') +''DS.''+ A.Name + ''= SR.''+ A.Name,'''')  
		FROM 
		(	SELECT 
			 c.name
			,c.is_identity
			,IIF(
			EXISTS(
			SELECT 1 FROM #DBS#.sys.index_COLUMNS IC 
			INNER JOIN #DBS#.sys.indexes I ON IC.index_id = I.index_id AND IC.object_id = I.object_id and I.is_primary_key = 1
			WHERE IC.object_id = C.Object_ID AND IC.column_id = c.column_id
			),1,0) AS is_primary_key

			FROM #DBS#.sys.columns c 
			INNER JOIN #DBS#.sys.tables t ON t.name = @TableX AND c.object_id = t.object_id 
			WHERE c.is_computed = 0
		) A'

    ,@Sablona_PARAMETRY NVARCHAR(max) = N'@CMDX NVARCHAR(MAX) OUT,@CMDXSet NVARCHAR(MAX) OUT,@CMDXOn NVARCHAR(MAX) OUT,@IdentityX INT OUT,@IDDisableX BIT,@TableX NVARCHAR(200)'

--Není li uvedeno, nastaví se aktuální databáze
    IF @SourceDB IS NULL  
        SET @SourceDB = DB_NAME() 


    -- Import/obnova tabulek
    IF (@IsExport = 0)
    BEGIN
        DECLARE ImportKurzor CURSOR LOCAL FAST_FORWARD FOR 
        SELECT DISTINCT 
        LTRIM(RTRIM(value))
        FROM STRING_SPLIT (@TableString, ',')
        

        OPEN ImportKurzor
        WHILE 1 = 1 
        BEGIN
            FETCH NEXT FROM ImportKurzor INTO @TableName
            IF @@FETCH_STATUS <> 0 
            BREAK
            
            SELECT
             @TableName = LTRIM(RTRIM(@TableName))
            ,@TableNameSfx = LTRIM(RTRIM(@TableName)) + @Suffix
            ,@ExistujeTabulka = 0
            ,@CMDSQL = N'SELECT @ExistujeTabulkaX=1 FROM '+@SourceDB+'.INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME = @TableNameX'
           
            EXECUTE sp_executesql @CMDSQL ,N'@ExistujeTabulkaX BIT OUT,@TableNameX NVARCHAR(100)',@ExistujeTabulkaX = @ExistujeTabulka OUT,@TableNameX =@TableNameSfx
           
            IF @ExistujeTabulka = 1
            BEGIN
                -- Seznam polí, které nejsou výpočtové
				PRINT @TableName
                SELECT 
                 @CMDSQL = REPLACE(@Sablona_POLE,@Sablona_DB,@DestinationDB )
                ,@CMDSQL2=''
                ,@Identity = 0
             
				-- Získání seznamu polí tabulky pro MERGE
                EXECUTE sp_executesql 
					@CMDSQL, 
					@Sablona_PARAMETRY,
					@CMDX = @CMDSQL2 OUT,
					@CMDXSet = @CMDSet OUT,
					@CMDXOn = @CMDOn OUT,
					@IdentityX = @Identity OUT,
					@IDDisableX= 1,
					@TableX = @TableName;
				
			
			    -- Konstrukce a spuštění MERGE
				SET @CMDSQL = 'MERGE ' + @DestinationDB + '.dbo.' + @TableName+ ' AS DS  USING ' + @SourceDB + '.dbo.'+ @TableNameSfx + ' AS SR ON ' + @CMDOn + ' WHEN MATCHED THEN UPDATE SET ' + @CMDSet + ' WHEN NOT MATCHED BY TARGET THEN INSERT (' + @CMDSQL2 + ')  VALUES (SR.' + REPLACE(@CMDSQL2,',',',SR.') + ') WHEN NOT MATCHED BY SOURCE THEN DELETE;'                    
                -- SELECT @TableName AS Tbl,@CMDSQL AS CMD
				EXEC(@CMDSQL)
            END    
        END
        CLOSE ImportKurzor
        DEALLOCATE ImportKurzor
    END

    -- Export tabulek
    IF( @IsExport = 1 ) 
    BEGIN
            DECLARE ExportKurzor CURSOR LOCAL FAST_FORWARD FOR
            
            SELECT DISTINCT 
			LTRIM(RTRIM(value))
            FROM STRING_SPLIT (@TableString, ',')

        OPEN ExportKurzor
        WHILE 1 = 1 
        BEGIN
            FETCH NEXT FROM ExportKurzor INTO @TableName
            IF @@FETCH_STATUS <> 0 
                BREAK
            

            SELECT
             @TableName = LTRIM(RTRIM(@TableName))
            ,@ExistujeTabulka = 0
            ,@CMDSQL = N'SELECT @ExistujeTabulkaX=1 FROM '+@SourceDB+'.INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME = @TableNameX'
           
             EXECUTE sp_executesql @CMDSQL ,N'@ExistujeTabulkaX BIT OUT,@TableNameX NVARCHAR(100)',@ExistujeTabulkaX = @ExistujeTabulka OUT,@TableNameX =@TableName
           
            IF @ExistujeTabulka = 1
            BEGIN
                -- Seznam polí, které nejsou výpočtové
                SET @CMDSQL = REPLACE(@Sablona_POLE,@Sablona_DB,@SourceDB )
                SET @CMDSQL2=''
                     
			    EXECUTE sp_executesql 
						@CMDSQL, 
						@Sablona_PARAMETRY,
						@CMDX = @CMDSQL2 OUT,
						@CMDXSet = @CMDSet OUT,
						@CMDXOn = @CMDOn OUT,
						@IdentityX = @Identity OUT,
						@IDDisableX= 0,
						@TableX = @TableName;
						
				-- Zrušení TMP Tabulek a Vytvoření nových se Suffixem.
			    SET @CMDSQL = 'DROP TABLE IF EXISTS '+@DestinationDB+'.dbo.'+@TableName+@Suffix+';SELECT '+@CMDSQL2+'  INTO '+@DestinationDB+'.dbo.'+@TableName+@Suffix+' FROM '+@SourceDB+'.dbo.'+ @TableName 
				 -- SELECT @TableName AS Tbl,@CMDSQL AS CMD
				EXEC(@CMDSQL)
            END
        END
        CLOSE ExportKurzor
        DEALLOCATE ExportKurzor

    END 
END


